# Author: Toshio Kuratomi <tkuratom@redhat.com>
# License: GPLv3+
# Copyright: Ansible Project, 2020


def generate_docs(args) -> int:
    raise NotImplementedError('Priority to implement subcommands is stable, devel, plugin, and'
                              ' then collection commands')
